// ============================================================================
// ==                                                                        ==
// == AMDiS - Adaptive multidimensional simulations                          ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  crystal growth group                                                  ==
// ==                                                                        ==
// ==  Stiftung caesar                                                       ==
// ==  Ludwig-Erhard-Allee 2                                                 ==
// ==  53175 Bonn                                                            ==
// ==  germany                                                               ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  http://www.caesar.de/cg/AMDiS                                         ==
// ==                                                                        ==
// ============================================================================

/** \file SmootherBase.h */

#include "Global.h"
#include <set>
#include <string>
#include "CreatorInterface.h"

#ifndef AMDIS_SMOOTHERBASE_H
#define AMDIS_SMOOTHERBASE_H

namespace AMDiS {

  template<typename MatrixType, typename VectorType, typename DOFSetType>
  class SmootherBase
  {
  public:
    SmootherBase(std::string name) : name_(name) {};

    virtual ~SmootherBase() {};

    virtual void smooth(MatrixType       *matrix,
			VectorType       *solution,
			VectorType       *rhs,
			int               iterations,
			const DOFSetType &dofSet) = 0;

  protected:
    const std::string name_;
  };


  template<typename MatrixType, typename VectorType, typename DOFSetType>
  class SmootherCreator : public CreatorInterface<SmootherBase<MatrixType, 
							       VectorType,
							       DOFSetType> >
  { 
  public:
    virtual ~SmootherCreator() {};

    void setName(std::string name_) { name = name_; };

  protected:
    std::string name;
  };

}

#endif
